import os
import pathlib
import re
import zipfile

from py7zr import Bad7zFile, SevenZipFile


class InvalidArchive(Exception):
    pass


class Archive:
    def __init__(self, file):
        try:
            self.archive = (
                zipfile.ZipFile(file, mode="r")
                if zipfile.is_zipfile(file)
                else SevenZipFile(file, mode="r")
            )
        except Bad7zFile:
            raise InvalidArchive

    @property
    def name(self):
        return pathlib.Path(self.archive.filename).stem

    @property
    def region(self):
        results = re.findall(r"\((.*?)\)", self.name)
        return results[0] if results else None

    def extract(self, dest):
        self.archive.extractall(path=os.path.join(dest, self.name))



import argparse
import multiprocessing

from .modetool import MODETool


def main():
    parser = argparse.ArgumentParser(description="MODE Tool")
    parser.add_argument(
        "-i", "--input", type=str, help="Directory containing compressed images."
    )
    parser.add_argument(
        "-o", "--output", type=str, help="Directory for extracting images too."
    )
    parser.add_argument(
        "-r", "--regions", type=str, help="Comma-separated list of regions to filter by."
    )
    parser.add_argument(
        "-w",
        "--workers",
        type=int,
        default=int(multiprocessing.cpu_count()),
        help="The number of worker threads to use.",
    )
    parser.add_argument(
        "-s", "--sort", help="Extract images to subdirectories based on region.", action="store_true",
    )

    args = parser.parse_args()

    regions = []

    if args.regions:
        regions = args.regions.split(",")

    mode = MODETool(
        input=args.input,
        output=args.output,
        regions=regions,
        workers=args.workers,
        sort=args.sort,
    )

    mode.run()



import itertools
import multiprocessing
import os

from loguru import logger

from .archive import Archive, InvalidArchive


class MODETool:
    def __init__(self, input, output, regions, workers, sort):
        self.input = input
        self.output = output
        self.regions = regions
        self.workers = workers
        self.sort = sort
        self.pool = None

    def worker(self, path):
        try:
            archive = Archive(path)
            dest = os.path.join(self.output, archive.region if self.sort else str())

            if self.regions and archive.region not in self.regions:
                return

            logger.info(f"Extracting: {dest}/{archive.name}")
            archive.extract(dest=dest)
        except InvalidArchive:
            return

    def run(self):
        try:
            logger.info("Running")
            logger.info(
                f"input={self.input} output={self.output} regions={self.regions} workers={self.workers} sort={self.sort}"
            )
            pool = multiprocessing.Pool(self.workers)
            walk = os.walk(self.input)
            jobs = itertools.chain.from_iterable(
                (os.path.join(root, file) for file in files)
                for root, dirs, files in walk
            )

            pool.map(self.worker, jobs)
        except KeyboardInterrupt:
            logger.info("Shutting down workers")
            pool.terminate()
            pool.join()
